% Compute an intercell flux according to the FORCE scheme.
%   _____                  
%  |  ___|__  _ __ ___ ___ 
%  | |_ / _ \| '__/ __/ _ \
%  |  _| (_) | | | (_|  __/
%  |_|  \___/|_|  \___\___|
%                        
%
%  Name of program: Force.m
%
%  Developed by: Enrico Bertolazzi
%                Department of Mechanical and Structural Engineering
%                enrico.bertolazzi@ing.unitn.it
%                http://www.ing.unitn.it/~bertolaz/
%
%  Last revision: 3 June 2008
%
%
%  Purpose: Compute the the Lax-Friedrichs numerical flux
%
%           / DX         \     1   /           /  DX          \                   /  DX         \ \
%    Force | ----, QL, QR | = --- | Richtmyer |  --- , QL, QR |  + LaxFriedrichs |  --- , QL, QR | |
%           \ DT         /     2   \           \  DT          /                   \  DT         / /
%
%  The fuction numFlux(DX/DT,QL,QR) compute the intercell flux between two cell of size DX
%  with left state QL and right stagte QR. Different numerical fluxes results in different
%  numerical schemes. Some other functions are used to complete the definition of the numerical
%  scheme.
%
%  The scheme can be visualized in the following way
% 
%   +===+  The cell where the average value Qi is "stored".
%
%   +...+  Ghost cell. These cells are used to extrapolate values for the
%          construction of the bounary conditions. Two ghost cell for baoundary
%          are enough for all the considered fluxes.
%
%     +    The nodal values of the boundary of the cells.
%
%   +....+....+====+====+====+====+====    =====+======+......+.....+
%   : Q  : Q  | Q  | Q  | Q  |    |         Q   | Q    | Q    : Q   :
%   :  -1:  0 |  1 |  2 |  3 |               N-1|  N   |  N+1 :  N+2:
%   +....+....+====+====+====+====+====    =====+======+......+.....+
%
%  MATLAB cannot use negative index so that index are shifted by 2.
%
%  INPUTS:
%
%     DXDT = The ratio DX/DT: the grid speed.
%
%     Flux = the function handle of the routine comuting the flux F(Q).
%            Using the command FQ = feval( Flux, Q ) then
%            FQ(:,i) contains Flux( Q(:,i) ) 
% 
%     Q    = a matrix m x (N+4) where m is the dimension of the problem
%            and N is the number of the cells.
%
%  OUTPUTS:
%
%     F   = a matrix m n (N+1) with the numerical cells boundary fluxes
%
%     F(:,k) contains the boundary flux between cells k-1 and k.
%
function F = Force( DXDT, Flux, Q )
  QL = Q(:,2:end-2) ;
  QR = Q(:,3:end-1) ;
  %%
  FL = feval( Flux, QL) ;
  FR = feval( Flux, QR) ;
  % Richtmyer
  QS  = 0.5 .* ( QL + QR + (FL-FR) / DXDT ) ;
  FRM = feval( Flux, QS ) ;
  % LaxFriedrichs
  FLF = 0.5 * ( FL + FR + DXDT * (QL-QR) ) ;
  F   = 0.5 .* ( FRM + FLF ) ;
end
