% Compute the Lax-Friedrichs numerical flux
%   _                    _____     _          _      _      _         
%  | |    __ ___  __    |  ___| __(_) ___  __| |_ __(_) ___| |__  ___ 
%  | |   / _` \ \/ /____| |_ | '__| |/ _ \/ _` | '__| |/ __| '_ \/ __|
%  | |__| (_| |>  <_____|  _|| |  | |  __/ (_| | |  | | (__| | | \__ \
%  |_____\__,_/_/\_\    |_|  |_|  |_|\___|\__,_|_|  |_|\___|_| |_|___/
%
%  Name of program: LaxFriedrichs.m
%
%  Developed by: Enrico Bertolazzi
%                Department of Mechanical and Structural Engineering
%                enrico.bertolazzi@ing.unitn.it
%                http://www.ing.unitn.it/~bertolaz/
%
%  Last revision: 3 June 2008
%
%
%  Purpose: Compute the the Lax-Friedrichs numerical flux
%
%                   / DX         \     1   /                  DX             \
%    LaxFriedrichs | ----, QL, QR | = --- |  F(QR) + F(QL) - ---- ( QR - QL ) |
%                   \ DT         /     2   \                  DT             /
%
%  The fuction numFlux(DX/DT,QL,QR) compute the intercell flux between two cell of size DX
%  with left state QL and right stagte QR. Different numerical fluxes results in different
%  numerical schemes. Some other functions are used to complete the definition of the numerical
%  scheme.
%
%  The scheme can be visualized in the following way
% 
%   +===+  The cell where the average value Qi is "stored".
%
%   +...+  Ghost cell. These cells are used to extrapolate values for the
%          construction of the bounary conditions. Two ghost cell for baoundary
%          are enough for all the considered fluxes.
%
%     +    The nodal values of the boundary of the cells.
%
%   +....+....+====+====+====+====+====    =====+======+......+.....+
%   : Q  : Q  | Q  | Q  | Q  |    |         Q   | Q    | Q    : Q   :
%   :  -1:  0 |  1 |  2 |  3 |               N-1|  N   |  N+1 :  N+2:
%   +....+....+====+====+====+====+====    =====+======+......+.....+
%
%  MATLAB cannot use negative index so that index are shifted by 2.
%
%  INPUTS:
%
%     DXDT = The ratio DX/DT: the grid speed.
%
%     Flux = the function handle of the routine comuting the flux F(Q).
%            Using the command FQ = feval( Flux, Q ) then
%            FQ(:,i) contains Flux( Q(:,i) ) 
% 
%     Q    = a matrix m x (N+4) where m is the dimension of the problem
%            and N is the number of the cells.
%
%  OUTPUTS:
%
%     F   = a matrix m n (N+1) with the numerical cells boundary fluxes
%
%     F(:,k) contains the boundary flux between cells k-1 and k.
%
function F = LaxFriedrichs( DXDT, Flux, Q )
  FQ = feval( Flux, Q ) ;
  %
  QL = Q (:,2:end-2) ;
  QR = Q (:,3:end-1) ;
  FL = FQ(:,2:end-2) ;
  FR = FQ(:,3:end-1) ;
  %
  F = 0.5 * ( FL + FR + DXDT * (QL-QR) ) ;
end
