% Driver to test HyperbolicFV 
%
%  Developed by: Enrico Bertolazzi
%                Department of Mechanical and Structural Engineering
%                enrico.bertolazzi@ing.unitn.it
%                http://www.ing.unitn.it/~bertolaz/
%
%  Last revision: 8 June 2008
%
%
%      /    \   /          \     /    \       /                                    \
%      | q1 |   |  0    -a |     | q1 |    1  | (beta1+beta2)*q1 +(beta1-beta2)*q2 |
%  D_t |    | + |          | D_x |    | = --- |                                    |
%      | q2 |   | -a     0 |     | q2 |    2  | (beta1-beta2)*q1 +(beta1+beta2)*q2 |
%      \    /   \          /     \    /       \                                    /
%
%  for t > 0 and a <= x <= b
%
%  This function contain all the information for the Hyperbolic Linear system.
%  Is programmed in object oriented way.
%  The first argument is a string with the name of the "method".
%  The variable argument "varargin" contains the arguments of the 
%  corresponding method.

function Linear2DExampleFV

  global Linear2DExampleNflux ;
  
  nflux = { 'LaxFriedrichs', 'Force', 'Richtmyer', 'GodunovCentered' } ;
  cfl   = { 0.95, 0.95, 0.95, 0.7 } ;

  apar  = 1 ;
  beta1 = 0.1 ;
  beta2 = -0.0 ;
  
  for i=1:4
      
    ok = input( ['Numerical solver: ' nflux{i} ' [q=quit] ' ], 's' ) ;
    if ok == 'q'
      break ;
    end ;
  
    close all ;  
      
    Linear2DExampleNflux = nflux{i} ;

    % setup the model
    Linear2DModel( 'setup', apar, beta1, beta2 ) ;

    % get names of variable
    names = Linear2DModel( 'get_names' ) ;

    % Initial left and right state
    QL = [0.5 ; 2   ];
    QR = [1   ; 0.5 ];
    N  = 40 ;
    a  = -50 ;
    b  = 50  ;

    Qinit     = [ QL * ones(1,N/2) QR * ones(1,N-N/2) ] ; % initial condition
    Xnodes    = [ a:(b-a)/N:b ] ; % boundary cells nodes
    FinalTime = 30  ; % final time
    StepTime  = 0.1 ; % save a frame every StepTime
    CFLratio  = cfl{i} ;

    % Exact solution of the riemann problem sampled of fine mesh
    Xe = [ a:(b-a)/1000:b ] ;
    Qe = Linear2DModel( 'riemann', QL, QR, FinalTime, Xe ) ;

    handleFig = figure( 'Position', [10 10 1000 800], 'color', 'w' ) ;

    % call the solver
    [ Qframes, Tframes ] = SolverFV( FinalTime,    ...
                                     StepTime,     ...
                                     CFLratio,     ...
                                     Xnodes,       ...
                                     Qinit,        ...
                                     @numFlux,     ...
                                     @Extrapolate, ...
                                     @Source,      ...
                                     @maxSpeed ) ;

    % call the solver
    [ QMUSCL, TMUSCL ] = SolverMUSCL( FinalTime,    ...
                                      StepTime,     ...
                                      CFLratio,     ...
                                      Xnodes,       ...
                                      Qinit,        ...
                                      @Flux,        ...
                                      @numForce,    ...
                                      @Extrapolate, ...
                                      @Reconstruct, ...
                                      @Source,      ...
                                      @maxSpeed ) ;
      
    % evalute cell center
    X = (Xnodes(1:end-1)+Xnodes(2:end))/2 ;

    % extract last frame
    Q  = Qframes(:,:,end) ;
    QM = QMUSCL(:,:,end) ;

    for j=1:2
      subplot(2,1,j);
      hold on ;
      plot(Xe, Qe(j,:), '-b',  'LineWidth', 2) ;
      plot(X,  Q(j,:),  '-ok', 'MarkerFaceColor','r');
      plot(X,  QM(j,:), '-sk', 'MarkerFaceColor','g');
      title( sprintf('CFL: %d NCELL: %d component %s: ',cfl{i},N,names{j}), ...
             'FontSize', 24, 'FontName', 'Arial' );
      legend('exact',nflux{i},'MUSCL',3) ;
      hold off ;
    end

    set(gcf, 'PaperPositionMode', 'auto')   % Use screen size
    print('-djpeg',sprintf('%s.jpg',nflux{i})) ;

  end
end

% local function needed by the solver

%
% Remap the method 'flux' of Linear2DModel to the function Flux
%
function F = Flux( Q )
  F = Linear2DModel( 'flux', Q ) ;
end

%
% Remap the method 'extrapolate' of Linear2DModel to the function Extrapolate
%
function Qextra = Extrapolate( Q )
  left_bc  = 'free' ; %'cyclic' ;
  right_bc = 'free' ; %'cyclic' ;
  Qextra   = Linear2DModel( 'extrapolate', Q, left_bc, right_bc ) ;
end

%
% Remap the method 'source' of Linear2DModel to the function Source
%
function SRC = Source( T, X, Q )
  SRC = Linear2DModel( 'source', T, X, Q ) ;
end

%
% Remap the method 'max_speed' of Linear2DModel to the function maxSpeed
%
function S = maxSpeed( Q )
  S = Linear2DModel( 'max_speed', Q ) ;
end

%
% Remap the numerical solvers to the function numFlux
%
function F = numFlux( DXDT, Q )
  global Linear2DExampleNflux ;

  switch Linear2DExampleNflux
    case 'LaxFriedrichs'   ; F = LaxFriedrichs( DXDT, @Flux, Q ) ;
    case 'Force'           ; F = Force( DXDT, @Flux, Q ) ;
    case 'Richtmyer'       ; F = Richtmyer( DXDT, @Flux, Q ) ;
    case 'GodunovCentered' ; F = GodunovCentered( DXDT, @Flux, Q ) ;
  end ;
end

%
% Force numerical flux for MUSCL
%
function F = numForce( DXDT, QL, QR )
  %%
  FL = feval( @Flux, QL) ;
  FR = feval( @Flux, QR) ;
  % Richtmyer
  QS  = 0.5 .* ( QL + QR + (FL-FR) / DXDT ) ;
  FRM = feval( @Flux, QS ) ;
  % LaxFriedrichs
  FLF = 0.5 * ( FL + FR + DXDT * (QL-QR) ) ;
  F   = 0.5 .* ( FRM + FLF ) ;
end

%
% Remap the function WENOSlopeLimiter to the function Reconstruct
%
function [QL,QR] = Reconstruct(Qextra)
  QC = Linear2DModel( 'to_characterisctic', Qextra ) ;
  for k=1:length(Qextra(:,1))
    SLOPE(k,:) = WENOSlopeLimiter( QC(k,:) ) ;
  end;
  QL = QC(:,2:end-1) - SLOPE / 2 ;
  QR = QC(:,2:end-1) + SLOPE / 2 ;
  QL = Linear2DModel( 'from_characterisctic', QL ) ;
  QR = Linear2DModel( 'from_characterisctic', QR ) ;
end
