% Driver to test HyperbolicFV 
%
%  Developed by: Enrico Bertolazzi
%                Department of Mechanical and Structural Engineering
%                enrico.bertolazzi@ing.unitn.it
%                http://www.ing.unitn.it/~bertolaz/
%
%  Last revision: 8 June 2008
%
%      /    \   /          \     /    \       /                        \
%      | q1 |   |  0    -a |     | q1 |    1  | (b1+b2)*q1 +(b1-b2)*q2 |
%  D_t |    | + |          | D_x |    | = --- |                        |
%      | q2 |   | -a     0 |     | q2 |    2  | (b1-b2)*q1 +(b1+b2)*q2 |
%      \    /   \          /     \    /       \                        /
%
%  for t > 0 and a <= x <= b
%
%  This function contain all the information for the Hyperbolic Linear system.
%  Is programmed in object oriented way.
%  The first argument is a string with the name of the "method".
%  The variable argument "varargin" contains the arguments of the 
%  corresponding method.

function Linear2DExampleMUSCL

  close all ;  
      
  % setup the model
  Linear2DModel( 'setup', 1, 0, 0 ) ;

  % get names of variable
  names = Linear2DModel( 'get_names' ) ;

  % Initial left and right state
  QL = [0.5 ; 2   ];
  QR = [1   ; 0.5 ];
  N  = 100 ;
  a  = -1 ;
  b  = 1 ;

  Qinit     = [ QL * ones(1,N/2) QR * ones(1,N-N/2) ] ; % initial condition
  Xnodes    = [ a:(b-a)/N:b ] ; % boundary cells nodes
  FinalTime = 0.5  ; % final time
  StepTime  = 0.01 ; % save a frame every StepTime
  CFLratio  = 0.7  ;

  % Exact solution of the riemann problem sampled of fine mesh
  Xe = [ a:(b-a)/1000:b ] ;
  Qe = Linear2DModel( 'riemann', QL, QR, FinalTime, Xe ) ;

  handleFig = figure( 'Position', [10 10 1000 800], 'color', 'w' ) ;

  % call the solver
  [ Qframes, Tframes ] = HyperbolicMUSCL( FinalTime,    ...
                                          StepTime,     ...
                                          CFLratio,     ...
                                          Xnodes,       ...
                                          Qinit,        ...
                                          @Flux,        ...
                                          @numFlux,     ...
                                          @Extrapolate, ...
                                          @Reconstruct, ...
                                          @Source,      ...
                                          @maxSpeed ) ;

  % evalute cell center
  X = (Xnodes(1:end-1)+Xnodes(2:end))/2 ;

  % extract last frame
  Q = Qframes(:,:,end) ;
  for j=1:2
    subplot(2,1,j);
    plot(Xe, Qe(j,:), '-b',  'LineWidth', 2, 'Color', 'green') ; hold on ;
    plot(X,  Q(j,:),  '-ok', 'MarkerFaceColor','r');
    title( [ 'Numerical Flux: MUSCL component: ' names{j}] , ...
           'FontSize', 24, 'FontName', 'Arial' );
  end
end

% local function needed by the solver

%
% Remap the method 'flux' of Linear2DModel to the function Flux
%
function F = Flux( Q )
  F = Linear2DModel( 'flux', Q ) ;
end

%
% Remap the method 'extrapolate' of Linear2DModel to the function Extrapolate
%
function Qextra = Extrapolate( Q )
  left_bc  = 'free' ; %'cyclic' ;
  right_bc = 'free' ; %'cyclic' ;
  Qextra   = Linear2DModel( 'extrapolate', Q, left_bc, right_bc ) ;
end

%
% Remap the method 'flux' of Linear2DModel to the function Flux
%
function [QL,QR] = Reconstruct(Qextra)
  for k=1:length(Qextra(:,1))
    SLOPE(k,:) = WENOSlopeLimiter( Qextra(k,:) ) ;
  end;
  QL = Qextra(:,2:end-1) - SLOPE / 2 ;
  QR = Qextra(:,2:end-1) + SLOPE / 2 ;
end

%
% Remap the method 'source' of Linear2DModel to the function Source
%
function SRC = Source( T, X, Q )
  SRC = Linear2DModel( 'source', T, X, Q ) ;
end

%
% Remap the method 'max_speed' of Linear2DModel to the function maxSpeed
%
function S = maxSpeed( Q )
  S = Linear2DModel( 'max_speed', Q ) ;
end

%
% Force numerical flux
%
function F = numFlux( DXDT, QL, QR )
  %%
  FL = feval( @Flux, QL) ;
  FR = feval( @Flux, QR) ;
  % Richtmyer
  QS  = 0.5 .* ( QL + QR + (FL-FR) / DXDT ) ;
  FRM = feval( @Flux, QS ) ;
  % LaxFriedrichs
  FLF = 0.5 * ( FL + FR + DXDT * (QL-QR) ) ;
  F   = 0.5 .* ( FRM + FLF ) ;
end
