% model for the Hyperbolic Linear system 
%
%  Developed by: Enrico Bertolazzi
%                Department of Mechanical and Structural Engineering
%                enrico.bertolazzi@ing.unitn.it
%                http://www.ing.unitn.it/~bertolaz/
%
%  Last revision: 3 June 2008
%
%
%      /    \   /          \     /    \       /                        \
%      | q1 |   |  0    -a |     | q1 |    1  | (b1+b2)*q1 +(b1-b2)*q2 |
%  D_t |    | + |          | D_x |    | = --- |                        |
%      | q2 |   | -a     0 |     | q2 |    2  | (b1-b2)*q1 +(b1+b2)*q2 |
%      \    /   \          /     \    /       \                        /
%
%  for t > 0 and a <= x <= b
%
%  This function contain all the information for the Hyperbolic Linear system.
%  Is programmed in object oriented way.
%  The first argument is a string with the name of the "method".
%  The variable argument "varargin" contains the arguments of the 
%  corresponding method.
%
%  method:      'setup'
%  input:       a, beta1m beta2
%  output:      NONE
%  description: setup the value of a for the model
%
%  method:      'get_dim'
%  input:       NONE
%  output:      dim
%  description: return the dimension of the model in this case 2
%
%  method:      'get_names'
%  input:       NONE
%  output:      names
%  description: return a cell array with the names of the states in this { 'q1', 'q2' }
%
%  method:      'flux'
%  input:       Q
%  output:      F(Q)
%  description: compute the flux of the model
%
%  method:      'source'
%  input:       T,X,Q
%  output:      S(T,Q)
%  description: compute the flux of the model
%
%  method:      'riemann'
%  input:       QL, QR, T, X
%  output:      Q
%  description: Compute the exact solution of the riemann problem with
%               left state QL and right state QR.
%               T is a real values where the solution is sampled.
%               X is a vector of X nodes where the solution is sampled.
%               Q is a matrix 2 x N with the samples solution.
%
%  method:      'max_speed'
%  input:       Q
%  output:      S
%  description: Given a vector os states Q compute the maximum
%               signal speed over all states Q.
%
%  method:      'extrapolate'
%  input:       Q, left_bc, right_bc
%  output:      Qextra
%  description: Given the states Q compute the left and right ghost cells
%               depending on the boundary condition left_bc, right_bc.
%               Possible values are:
%               'free'   for tranparent boundary condition
%               'cyclic' for cyclic boundary condition
%
function [varargout] = Linear2DModel( what, varargin )

  global Linear2x2_paramaters ;

  switch lower( what )

    case 'setup'
      expected( what, 3, nargin, 0, nargout ) ; % a
      Linear2x2_paramaters = clear_struct(Linear2x2_paramaters) ;
      Linear2x2_paramaters . a     = varargin{1} ;
      Linear2x2_paramaters . beta1 = varargin{2} ;
      Linear2x2_paramaters . beta2 = varargin{3} ;

    case 'get_dim'
      expected( what, 0, nargin, 1, nargout ) ;
      varargout{1} = 2 ;

    case 'get_names'
      expected( what, 0, nargin, 1, nargout ) ;
      varargout{1} = { 'q1', 'q2' } ;

    case 'flux'
      expected( what, 1, nargin, 1, nargout ) ; % Q
      varargout{1} = Flux( varargin{1} ) ;

    case 'riemann'
      expected( what, 4, nargin, 1, nargout ) ; % QL, QR, T, X
      QL = varargin{1} ;
      QR = varargin{2} ;
      T  = varargin{3} ;
      X  = varargin{4} ;
      varargout{1} = RiemannExact( QL, QR, T, X ) ;

    case 'max_speed'
      expected( what, 1, nargin, 1, nargout ) ; % Q 
      varargout{1} = maxSpeed( varargin{1} ) ;

    case 'extrapolate'
      expected( what, 3, nargin, 1, nargout ) ; % Q, left_bc, right_bc
      Q        = varargin{1} ;
      left_bc  = varargin{2} ;
      right_bc = varargin{3} ;
      varargout{1} = Extrapolate( Q, left_bc, right_bc ) ;

    case 'source'
      expected( what, 3, nargin, 1, nargout ) ; % T, X, Q
      T = varargin{1} ;
      X = varargin{2} ;
      Q = varargin{3} ;
      varargout{1} = Source( T, X, Q ) ;

    case 'to_characterisctic'
      expected( what, 1, nargin, 1, nargout ) ; % Q
      Q = varargin{1} ;
      L = [ 0.5, 0.5 ; -0.5, 0.5 ] ;
      varargout{1} = L * Q ;

    case 'from_characterisctic'
      expected( what, 1, nargin, 1, nargout ) ; % Q
      Q = varargin{1} ;
      L = [ 0.5, 0.5 ; -0.5, 0.5 ] ;
      varargout{1} = L \ Q ;

    otherwise
      error(['Linear2x2Model bad argument: ', what]) ;
  end
end

function SE = clear_struct(S)
  if isstruct(S)
    SE = rmfield( S, fieldnames(S) ) ;
  else
    SE = S ;
  end
end

function expected( what, ein, nin, eout, nout )
  if ein ~= nin-1 
    error( ['in Linear2x2 call to ', what, ' expected ', int2str( ein ), ' input argument, found ', int2str(nin-1) ]) ;
  end
  if eout ~= nout 
    error( ['in Linear2x2 call to ', what, ' expected ', int2str( eout ), ' output argument, found ', int2str(nout) ]) ;  
  end
end  

%  #######                      
%  #       #      #    # #    # 
%  #       #      #    #  #  #  
%  #####   #      #    #   ##   
%  #       #      #    #   ##   
%  #       #      #    #  #  #  
%  #       ######  ####  #    # 
% compute the flux
function F = Flux( QS )
  global Linear2x2_paramaters ;
  a = Linear2x2_paramaters . a ;
  F = [ 0, -a ; -a, 0 ] * QS ;
end

%   #####                                     
%  #     #  ####  #    # #####   ####  ###### 
%  #       #    # #    # #    # #    # #      
%   #####  #    # #    # #    # #      #####  
%        # #    # #    # #####  #      #      
%  #     # #    # #    # #   #  #    # #      
%   #####   ####   ####  #    #  ####  ###### 
% compute the source
function S = Source( T, X, Q )
  global Linear2x2_paramaters ;
  beta1 = Linear2x2_paramaters . beta1 ;
  beta2 = Linear2x2_paramaters . beta2 ;
  cf1   = (beta1+beta2)/2 ;
  cf2   = (beta1-beta2)/2 ;
  S     = [ cf1 * Q(1,:) + cf2 * Q(2,:) ; cf2 * Q(1,:) + cf1 * Q(2,:) ] ;
end

%  ######                                       
%  #     # # ###### #    #   ##   #    # #    # 
%  #     # # #      ##  ##  #  #  ##   # ##   # 
%  ######  # #####  # ## # #    # # #  # # #  # 
%  #   #   # #      #    # ###### #  # # #  # # 
%  #    #  # #      #    # #    # #   ## #   ## 
%  #     # # ###### #    # #    # #    # #    # 
%
function QS = RiemannStar( T, QL, QR )
  global Linear2x2_paramaters ;
  beta1 = Linear2x2_paramaters . beta1 ;
  beta2 = Linear2x2_paramaters . beta2 ;
  A  = ((QR(1,:)+QR(2,:))/2).*exp(beta1*T) ;
  B  = ((QL(1,:)-QL(2,:))/2).*exp(beta2*T)  ;
  QS(1,:) = A + B ;
  QS(2,:) = A - B ;
end

function QS = RiemannExact( QL, QR, T, X )
  global Linear2x2_paramaters ;
  a = Linear2x2_paramaters . a ;

  l1 = -abs(a) ;
  l2 = +abs(a) ;
  
  IDX = find( X - l1 * T >= 0 & X - l2 * T <= 0 ) ;
  QS(:,IDX) = RiemannStar( T, QL, QR ) * ones( size(IDX) ) ;

  IDX = find( X - l1 * T < 0 ) ;
  QS(:,IDX) = RiemannStar( T, QL, QL ) * ones( size(IDX) ) ;

  IDX = find( X - l2 * T > 0 ) ;
  QS(:,IDX) = RiemannStar( T, QR, QR ) * ones( size(IDX) ) ;
end

%                        #####                              
%  #    #   ##   #    # #     # #####  ###### ###### #####  
%  ##  ##  #  #   #  #  #       #    # #      #      #    # 
%  # ## # #    #   ##    #####  #    # #####  #####  #    # 
%  #    # ######   ##         # #####  #      #      #    # 
%  #    # #    #  #  #  #     # #      #      #      #    # 
%  #    # #    # #    #  #####  #      ###### ###### ##### 
% 
function MS = maxSpeed( Q )
  global Linear2x2_paramaters ;
  a  = Linear2x2_paramaters . a ;
  MS = abs(a) ;
end

%  #######                                                                     
%  #       #    # ##### #####    ##   #####   ####  #        ##   ##### ###### 
%  #        #  #    #   #    #  #  #  #    # #    # #       #  #    #   #      
%  #####     ##     #   #    # #    # #    # #    # #      #    #   #   #####  
%  #         ##     #   #####  ###### #####  #    # #      ######   #   #      
%  #        #  #    #   #   #  #    # #      #    # #      #    #   #   #      
%  ####### #    #   #   #    # #    # #       ####  ###### #    #   #   ###### 
%
function Qextra = Extrapolate( Q, L, R )

  global Linear2x2_paramaters ;
  a = Linear2x2_paramaters . a ;

  % Left boundary
  switch lower(L)
    case 'free'   % Apply transmissive boundary conditions
      QL  = Q(:,1) ;
      QLL = Q(:,2) ;
    case 'cyclic' % Apply cyclic boundary conditions
      QL  = Q(:,end) ;
      QLL = Q(:,end-1) ;
    otherwise
      error( ['in Linear2x2 call to Extrapolate L=', L, ' unknown boundary condition']) ;  
  end
  % Right boundary
  switch lower(R)
    case 'free'   % Apply transmissive boundary conditions
      QR  = Q(:,end) ;
      QRR = Q(:,end-1) ;
    case 'cyclic' % Apply cyclic boundary conditions
      QR  = Q(:,1) ;
      QRR = Q(:,2) ;
    otherwise
      error( ['in Linear2x2 call to Extrapolate R=', R, ' unknown boundary condition']) ;  
  end

  % build vector of extrapolated values
  Qextra = [ QLL, QL, Q, QR, QRR ] ;
end
