%  Numerical scheme for a Nonlinear System of One Dimensional Conservation Law
%  Name of program: SolverFV.m
%
%  Developed by: Enrico Bertolazzi
%                Department of Mechanical and Structural Engineering
%                enrico.bertolazzi@ing.unitn.it
%                http://www.ing.unitn.it/~bertolaz/
%
%  This code is inspired on the library
%    NUMERICA
%    A Library of Source Codes for Teaching,
%    Research and Applications,
%    by E. F. Toro
%    Published by NUMERITEK LTD,
%    Website: www.numeritek.coM
%
%  The algorithm are described in the book:
%
%    E. F. Toro, "Riemann Solvers and Numerical Methods for Fluid Dynamics"
%                 Springer-Verlag, 1997, Second Edition, 1999
%
%  Last revision: 3 June 2008
%
%
%  Purpose: Compute the numerical solution of the hyperbolic problem
%
%    Q (t,x) + F (Q(t,x)) = S(t,x,Q(t,x))
%     t         x
%
%  with initial data
%
%    Q(0,x) = Qinit(x)  x in [a,b]
%
%  the boundary condition are determined by the Extrapolate(Q) function.
%  
%  The problem is solved by finite volume update formula
%
%     k+1    k     DT     /                  \
%    Q    = Q   - ---- * |  F      - F       |  + DT S
%     i      i     DX     \  i+1/2    i-1/2  /        i
%
%  where F       is the numerical flux for the boundary between cell i and i+1
%         i+1/2
%
%  For the majority of fluxes is something like
%
%                / DX             \
%    F      = F | ----, Q  , Q     |
%     i+1/2      \ DT    i    i+1 /
%
%  But some second order fluxes (e.g. Beam-Warming) needs a larger stencils
%  and are something like
%
%                / DX                         \
%    F      = F | ----, Q   , Q  , Q   , Q     |
%     i+1/2      \ DT    i-1   i    i+1   i+2 /
%
%  S  is the contribution of the source term
%   i
%                /  k       k \
%    S      = S |  t, x,  Q   |
%     i          \     i   i /
%
%         k                                                                 k
%  where Q  correspond to the cell average approximation of cell i at time t.
%         i
%
%  The scheme can be visualized in the following way
% 
%   +===+  The cell where the average value Qi is "stored".
%
%   +...+  Ghost cell. These cells are used to extrapolate values for the
%          construction of the bounary conditions. Two ghost cell for baoundary
%          are enough for all the considered fluxes.
%
%     +
%     |    The boundary between two cells, where the intercell fluxes are defined.
%     + 
%
%     +    The nodal values of the boundary of the cells.
%
%
%
%   +...+...+===+===+===+===+===    ===+===+...+...+
%           |   |   |   |   |          |   |
%   +...+...+===+===+===+===+===    ===+===+...+...+   T(3)
%           |   |   |   |   |          |   |   |   |
%   +...+...+===+===+===+===+===    ===+===+...+...+   T(2)
%           |   |   |   |   |          |   |
%   +...+...+===+===+===+===+===    ===+===+...+...+   T(1)
%             Q1  Q2  Q3                 QN
%
%
%
%  INPUTS:
%
%     FinalTime   = The numerical solution is computed from t=0 up to t=FinalTime
%     StepTime    = every StepTime the solution is saved in the Qout Tout matrices.
%     CFLratio    = the advancing step is given as a ratio of the CFL number.
%
%     Xnodes      = a vector of N+1 values, the boundary of the cells.
%     Qinit       = a matrix m x N where m is the dimension of the problem
%                   and N is the number of the cells.
%
%     numFlux     = the function handle of the numerical flux. It accept 2 arguments, for example
%
%                   RES = numFlux(DX/DT, Q)
%
%                   DX/DT the grid velocity, some numerical fluxes need it
%                   Q  a matrix m x (N+4) which contains N+4 states. The first and last 2 columns
%                      contain the extrapolated values.
%
%                   the columns of RES contains the computed boundary fluxes, i.e.
%
%                   for the majority of fluxes is something like
%                   RES(:,k) = FLUX( DX/DT, Q(:,k+1), Q(:,k+2) )                    k=1,2,...,N+1
%
%                   for some high order fluxes is something like
%                   RES(:,k) = FLUX( DX/DT, Q(:,k), Q(:,k+1), Q(:,k+2), Q(:,k+3) )  k=1,2,...,N+1
%
%                   RES(:,k) is the boundary fluxes between cell k-1 and k
%
%     Extrapolate = the function handle of the extrapolate function.
%                   It extrapolate values for ghost cells depending of boundary conditions.
%                   Accept a matrix m x N and return an m x (N+4) matrix with addedd the
%                   extrapolated values. For example
%
%                   Qextra = feval( Extrapolate, Q )
%
%                   then Qextra(:,k+2) = Q(:,k) for k=1,2,...,N
%                   Qextra(:,1), Qextra(:,2), Qextra(:,N+3), Qextra(:,N+4) extrapolated values.
%
%     Source      = the function handle of the source function.
%                   It compute the source term.
%                   Accept a matrix m x N and return an m x (N+4) matrix with addedd the
%                   extrapolated values. For example
%
%                   S = feval( Source, T, Q )
%
%                   T is a scalar
%                   Q is a matrix m x N with the states on the cells
%
%     maxSpeed    = the function handle of the maximum signal speed function.
%                   It compute tghe maximum speed used for compute the time step DT.
%
%                   S = maxSpeed(Q)
%
%                   Q a matrix m x P which contains P states. S is the maximum speed signal
%
%  OUTPUTS:
%
%     Tframes     = contains the time at which the solution is saved.
%                   Tframes(1)   = 0
%                   Tframes(2)   = StepTime
%                   Tframes(3)   = 2*StepTime
%                   ...
%                   Tframes(end) = FinalTime
%
%     Qframes     = contains the computed discrete solution at discrete time.
%                   Qframes(:,:,k) contains the matrix m x N of the discrete sution at time Tframe(k)
%
%
function [ Qframes, Tframes ] = SolverFV( FinalTime,   ...
                                          StepTime,    ...
                                          CFLratio,    ...
                                          Xnodes,      ...
                                          Qinit,       ...
                                          numFlux,     ...
                                          Extrapolate, ...
                                          Source,      ...
                                          maxSpeed )

  % Initial conditions are copied in the vector Q
  Q = Qinit ;

  % Compute cell size
  DX = Xnodes(2) - Xnodes(1) ; % equal spaced nodes!!

  % compute cell centers
  X  = (Xnodes(1:end-1) + Xnodes(2:end)) / 2 ;

  % save initial frame
  Qframes(:,:,1) = Q ;
  Tframes(1)     = 0 ;

  kframe   = 1        ; % number of saved framed 
  Time     = 0        ; % initial time
  NextTime = StepTime ; % Time to save next frame

  % Time marching procedure, a maximum of 10000 steps are allowed
  for NITER=1:10000

    % Courant-Friedrichs-Lewy (CFL) condition imposed
    DT = DX * CFLratio / feval( maxSpeed, Q ) ;

    % update time
    Time = Time + DT ;

    % adjust time step if necessary to match the time grid
    % 
    % If Time is greater than NextTime than a step of the right length 
    % to match NextTime is computed. To avoid excess of numerical diffusion
    % (to small steps introduce many numerical diffusion) the computed solution
    % is saved but the next step start from the previous computed solution.
    %
    while Time >= NextTime
      % reduce DT in order to match NextTime
      DT1 = DT - ( Time - NextTime ) ;
      
      % perform step
      Qnew = doStep( Time, DT1, DX, X, Q, numFlux, Extrapolate, Source ) ;

      % Solution frame is saved
      kframe              = kframe+1 ;
      Qframes(:,:,kframe) = Qnew ;
      Tframes(kframe)     = NextTime ;

      % update next time frame
      NextTime = NextTime + StepTime ;
    end ;

    % perform step
    Q = doStep( Time, DT, DX, X, Q, numFlux, Extrapolate, Source ) ;

    % chek if all values are computed
    if Time >= FinalTime
      break ;
    end ;
  end ;
  
  % save last frame
  kframe              = kframe+1 ;
  Qframes(:,:,kframe) = Q ;
  Tframes(kframe)     = Time ;
end

%
% perform a forward step
%
function Qnew = doStep( Time, DT, DX, X, Q, numFlux, Extrapolate, Source )

  % Extrapolate for Boundary conditions
  Qextra = feval( Extrapolate, Q ) ;

  % Compute intercell flux FLUX
  FLUX = feval( numFlux, DX/DT, Qextra ) ;

  % Compute source term
  SRC  = feval( Source, Time, X, Q ) ;

  % Solution is updated according to conservative formula
  Qnew = Q + (DT/DX) * ( FLUX(:,1:end-1) - FLUX(:,2:end) ) + DT * SRC ;

end
